<!doctype html>
<html lang="en">
<head>
  <meta http-equiv="Content-Type" content="text/html; charset=utf-8">
  <meta name="viewport" content="width=device-width, initial-scale=1.0">
  <title>Test template</title>
  <style>
    body
    {
      background-color: white;
      font-family: Lucida Grande,Lucida Sans Unicode,Lucida Sans,Geneva,Verdana,sans-serif;
    }

    div.document
    {
        max-width: 820px;
        top: 20px;
        overflow: visible;
        margin: 0 auto;
        width: 90%;
        padding-bottom: 50px;
    }

    div h1
    {
        font-size: 32px;
        font-weight: normal;
        margin-bottom: 10px;
        margin-top: 24px;
        color: black;
    }

    div h2
    {
        font-size: 24px;
        font-weight: normal;
        margin-bottom: 10px;
        margin-top: 24px;
        color: black;
    }

    div.figure {
        text-align: center;
    }

    .equation svg {
        margin-left: -5%;
        max-width: 110%;
        height: auto;
        object-fit: contain;
    }

    .inline-equation svg {
        vertical-align: middle;
    }

    .equation-container {
            display: flex;
            justify-content: space-between;
            align-items: center;
            width: 100%;
    }

    .equation {
        text-align: center;
        width: 100%;
    }

    .equation-number {
        text-align: right;
    }

    table {
        border-collapse: collapse;
        margin: 20px auto;
    }


    table th,
    table td {
        padding: 8px;
        font-variant-numeric: tabular-nums;
    }

    table th {
        background-color: #f4f4f4;
        font-weight: bold;
    }

    /* Alternating row colors for readability */
    table tr:nth-child(even) {
        background-color: #f9f9f9;
    }

    table tr:nth-child(odd) {
        background-color: #ffffff;
    }

    /* Caption styling */
    table caption {
        caption-side: top;
        font-weight: bold;
        font-size: 16px;
        margin-bottom: 10px;
    }

    /* Fine-tuning text in table */
    table td, table th {
        vertical-align: middle;
        line-height: 1.5;
    }
  </style>
</head>
<body>
    <div class="document">
        <!--CONTENT-->
    </div>
</body>